/**
 * Copyright (c) 2025 Enzien Audio, Ltd.
 * 
 * Redistribution and use in source and binary forms, with or without modification,
 * are permitted provided that the following conditions are met:
 * 
 * 1. Redistributions of source code must retain the above copyright notice,
 *    this list of conditions, and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the phrase "powered by heavy",
 *    the heavy logo, and a hyperlink to https://enzienaudio.com, all in a visible
 *    form.
 * 
 *   2.1 If the Application is distributed in a store system (for example,
 *       the Apple "App Store" or "Google Play"), the phrase "powered by heavy"
 *       shall be included in the app description or the copyright text as well as
 *       the in the app itself. The heavy logo will shall be visible in the app
 *       itself as well.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 */

#ifndef _HEAVY_CONTEXT_WSTD_FLDR_HPP_
#define _HEAVY_CONTEXT_WSTD_FLDR_HPP_

// object includes
#include "HeavyContext.hpp"
#include "HvSignalLine.h"
#include "HvControlBinop.h"
#include "HvSignalEnvelope.h"
#include "HvSignalVar.h"
#include "HvControlIf.h"
#include "HvControlCast.h"
#include "HvMath.h"
#include "HvControlVar.h"

class Heavy_WSTD_FLDR : public HeavyContext {

 public:
  Heavy_WSTD_FLDR(double sampleRate, int poolKb=10, int inQueueKb=2, int outQueueKb=0);
  ~Heavy_WSTD_FLDR();

  const char *getName() override { return "WSTD_FLDR"; }
  int getNumInputChannels() override { return 2; }
  int getNumOutputChannels() override { return 2; }

  int process(float **inputBuffers, float **outputBuffer, int n) override;
  int processInline(float *inputBuffers, float *outputBuffer, int n) override;
  int processInlineInterleaved(float *inputBuffers, float *outputBuffer, int n) override;

  int getParameterInfo(int index, HvParameterInfo *info) override;
  struct Parameter {
    struct In {
      enum ParameterIn : hv_uint32_t {
        FLDR = 0xE42C892E, // Fldr
        LMTR = 0xCB23DC60, // Lmtr
        MIX = 0x97D46AC6, // Mix
      };
    };
  };

 private:
  HvTable *getTableForHash(hv_uint32_t tableHash) override;
  void scheduleMessageForReceiver(hv_uint32_t receiverHash, HvMessage *m) override;

  // static sendMessage functions
  static void cBinop_mVs2igpZ_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_zqlLB81r_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_xU03PF82_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_KT5JLzk4_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cCast_raqLpEpf_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cCast_94povccD_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_Y9513FCQ_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cMsg_DzrhPvBF_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_aqW1jtp0_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_Dm7KhrS9_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_CWv7EQqK_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cVar_zbh465NE_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cCast_YuvGRWRj_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cVar_dGOvyl9o_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cVar_57TDzYgd_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cIf_VbvpFabg_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cIf_c1Q0GWFO_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void sEnv_j809TR6C_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void sEnv_zumsTAK2_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_jIxNCaqH_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cMsg_VbjQEKWv_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_1AyEM9Oa_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_sb0MtZyR_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cCast_7906ESEb_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cCast_SmzSmXOY_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_ufoRxQ1x_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cMsg_R42L2bnR_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_6WG3D5pb_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cCast_um3TvVbf_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cCast_vgocSFtt_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cCast_X2o15bmb_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cVar_UWrftw5W_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cIf_iPztEjCY_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cIf_v6lh6kTu_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void sEnv_5A4fFX3g_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void sEnv_la6aUdMv_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_y9hbWNRQ_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cMsg_j5OdC41D_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_13hpUzv3_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_KzjIYi55_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cCast_fPAeUQGr_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cCast_0g0djEHG_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_nQyP7Yfz_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cMsg_Dt4V09Qs_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_zQWMGNVJ_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cCast_NcGyDxZb_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cCast_AN6b4AUE_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cCast_xSJy6l2n_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cVar_sb6JFlWG_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_PoRJXQmO_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cMsg_eP4rdCm8_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cMsg_KANyEMlr_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cSend_K5FlpIIN_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_rp1LKaUG_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cMsg_W4C6S1Jq_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cMsg_sPStAcBu_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cMsg_t60EVkmH_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cReceive_WJXP4PNW_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cReceive_GjV4j1g7_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cReceive_tjpIavq9_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cReceive_YLOOduML_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cReceive_gRQmajnT_sendMessage(HeavyContextInterface *, int, const HvMessage *);

  // objects
  SignalLine sLine_mVvoXdlU;
  SignalEnvelope sEnv_j809TR6C;
  SignalEnvelope sEnv_zumsTAK2;
  SignalEnvelope sEnv_5A4fFX3g;
  SignalEnvelope sEnv_la6aUdMv;
  SignalLine sLine_6LyE894I;
  SignalLine sLine_CeELL4xP;
  ControlBinop cBinop_mVs2igpZ;
  ControlBinop cBinop_zqlLB81r;
  ControlBinop cBinop_xU03PF82;
  ControlBinop cBinop_KT5JLzk4;
  ControlBinop cBinop_Y9513FCQ;
  ControlBinop cBinop_aqW1jtp0;
  ControlBinop cBinop_Dm7KhrS9;
  ControlBinop cBinop_CWv7EQqK;
  ControlVar cVar_zbh465NE;
  ControlVar cVar_dGOvyl9o;
  SignalVarf sVarf_Msvs6cdm;
  ControlVar cVar_57TDzYgd;
  ControlIf cIf_VbvpFabg;
  ControlIf cIf_c1Q0GWFO;
  ControlBinop cBinop_jIxNCaqH;
  ControlBinop cBinop_1AyEM9Oa;
  ControlBinop cBinop_sb0MtZyR;
  ControlBinop cBinop_ufoRxQ1x;
  ControlBinop cBinop_6WG3D5pb;
  SignalVarf sVarf_IGUeoqTh;
  ControlVar cVar_UWrftw5W;
  ControlIf cIf_iPztEjCY;
  ControlIf cIf_v6lh6kTu;
  ControlBinop cBinop_y9hbWNRQ;
  ControlBinop cBinop_13hpUzv3;
  ControlBinop cBinop_KzjIYi55;
  ControlBinop cBinop_nQyP7Yfz;
  ControlBinop cBinop_zQWMGNVJ;
  ControlVar cVar_sb6JFlWG;
  ControlBinop cBinop_PoRJXQmO;
  SignalVarf sVarf_rdrLk6OW;
  SignalVarf sVarf_zdd8IFXs;
  SignalVarf sVarf_mkauxat9;
  SignalVarf sVarf_iVWElot3;
  ControlBinop cBinop_rp1LKaUG;
};

#endif // _HEAVY_CONTEXT_WSTD_FLDR_HPP_
